@@ -1,4 +1,4 @@
-# Copyright 2015-2019 D.G. MacCarthy <http://dmaccarthy.github.io>
+# Copyright 2015-2020 D.G. MacCarthy <http://dmaccarthy.github.io>
 #
 # This file is part of "sc8pr".
 #
@@ -16,14 +16,24 @@
 # along with "sc8pr".  If not, see <http://www.gnu.org/licenses/>.
 
 
-from math import hypot, pi, sin, cos, asin, atan2, floor
+from math import hypot, pi, sin, cos, asin, atan2, floor, sqrt
 
 DEG = pi / 180
 
+try: from math import prod
+except:
+    def prod(iterable, start=1):
+        for x in iterable: start *= x
+        return start
+
 def positiveAngle(a):
     "Return an angle between 0 and 360"
     return a - 360 * floor(a / 360)
 
+def smallAngle(a):
+    "Return an angle between -180 and 180"
+    return positiveAngle(a + 180) - 180
+
 def angleDifference(a2, a1=0):
     "Difference between 2 directions; [-180,180)"
     return positiveAngle(a2 - a1 + 180) - 180
@@ -32,10 +42,6 @@ def dist(p1, p2):
     "Distance between two points"
     return hypot(p2[0] - p1[0], p2[1] - p1[1])
 
-def sprod(v1, v2):
-    "2D scalar product"
-    return v1[0] * v2[0] + v1[1] * v2[1]
-
 def sigma(*args):
     "Sum one or more vectors"
     return tuple(sum(x[i] for x in args) for i in range(len(args[0])))
@@ -54,6 +60,22 @@ def delta(v2, v1=None, mag=None):
             y *= mag
     return x, y
 
+# 2D Vector multiplications
+def neg(v): return -v[0], -v[1]
+def smult(s, v):return s * v[0], s * v[1]
+def sprod(v1, v2): return v1[0] * v2[0] + v1[1] * v2[1]
+def vmult(v1, v2): return v1[0] * v2[0], v1[1] * v2[1]
+def vprod(v1, v2): return v1[0] * v2[1] - v1[1] * v2[0]
+
+def shiftAlongNormal(x, y, deriv, dr):
+    m = deriv(x, y)
+    if m == 0: dx, dy = 0, dr
+    else:
+        m = -1 / m
+        dx = (-dr if m > 0 else dr) / sqrt(1 + m*m)
+        dy = m * dx
+    return x + dx, y + dy
+
 def vec2d(r, a, deg=True):
     "2D Polar to Cartesian conversion"
     if deg: a *= DEG
@@ -106,3 +128,18 @@ def rotatedSize(w, h, angle):
     w = max(abs(pt[0]) for pt in pts)
     h = max(abs(pt[1]) for pt in pts)
     return w, h
+
+def circle_intersect(c1, r1, c2, r2):
+    "Find the intersection(s) of two circles as list of points"
+    d = dist(c1, c2)
+    if d > r2 + r1 or d == 0 or d < abs(r2 - r1): return []
+    r_sq = r2 * r2
+    x = (d*d + r_sq - r1 * r1) / (2*d)
+    ux, uy = delta(c1, c2, 1)
+    x0, y0 = c2
+    x0 += x * ux
+    y0 += x * uy
+    if x < r2:
+        y = sqrt(r_sq - x*x)
+        return [(x0 - y * uy, y0 + y * ux), (x0 + y * uy, y0 - y * ux)]
+    else: return [(x0, y0)]
