@@ -1,4 +1,4 @@
-# Copyright 2015-2019 D.G. MacCarthy <http://dmaccarthy.github.io>
+# Copyright 2015-2021 D.G. MacCarthy <http://dmaccarthy.github.io>
 #
 # This file is part of "sc8pr".
 #
@@ -16,73 +16,23 @@
 # along with "sc8pr".  If not, see <http://www.gnu.org/licenses/>.
 
 
-from math import log, exp
 import pygame
+from sys import stderr
 from sc8pr import Renderable, Image, Graphic, BaseSprite
-from sc8pr.shape import Shape
+from sc8pr._cs import _lrbt, makeCS
+from sc8pr.shape.locus import locus, Locus as _Locus
 from sc8pr.util import rgba, rangef
 from sc8pr.geom import rotatedSize, transform2dGen
 from sc8pr.text import Text
+from sc8pr.plot.regression import leastSq, expon, power
+
+print("WARNING: sc8pr.misc.plot is deprecated.", file=stderr)
 
 
-def _lrbt(lrbt, w, h):
-    "Calculate coordinate system limits"
-    n = len(lrbt)
-    if n < 4:
-        if not isinstance(lrbt, list): lrbt = list(lrbt)
-        dy = h * (lrbt[1] - lrbt[0]) / w
-        if n == 2:
-            dy /= 2
-            lrbt = lrbt + [-dy, dy]
-        else: lrbt = lrbt + [lrbt[2] + dy]
-    else: lrbt = lrbt[:4]
-    return lrbt
-
-def coordTr(lrbt, size):
+def coordTr(lrbt, size, invert=False):
     "Create a transformation for the given coordinate system"
-    l, r = lrbt[:2]
-    sx = size[0] / (r - l)
-    dx = sx * l
-    b, t = lrbt[2:]
-    sy = size[1] / (b - t)
-    dy = sy * t
-    return lambda p: (sx * p[0] - dx, sy * p[1] - dy)
+    return makeCS(lrbt, size)[0 if invert else 1]
 
-def locus(func, param, **kwargs):
-    "Generate a parameterized sequence of 2D points"
-    t0, t1, steps = param
-    for i in range(steps + 1):     
-        try:
-            x = t0 + i * (t1 - t0) / steps
-            try: y = func(x, **kwargs)
-            except: y = func(x)
-            yield y if type(y) in (list, tuple) else (x, y)
-        except: pass
-
-def leastSq(x, y):
-    "Perform a simple least squares linear regression"
-    n = len(x)
-    if len(y) != n: raise ValueError("x and y data must be the same size")
-    xav = sum(x) / n
-    yav = sum(y) / n
-    m = sum((x[i] - xav) * (y[i] - yav) for i in range(n))
-    m /= sum((xi - xav) ** 2 for xi in x)
-    b = yav - m * xav
-    return (lambda x: m * x + b), (m, b)
-
-def power(x, y):
-    "Least squares fit to model y = a x**n"
-    x = [log(xi) for xi in x]
-    y = [log(xi) for xi in y]
-    n, a = leastSq(x, y)[1]
-    a = exp(a)
-    return (lambda x:a * x**n), (a, n)
-
-def expon(x, y):
-    "Least squares fit to model y = a b**x"
-    y = [log(xi) for xi in y]
-    b, a = [exp(a) for a in leastSq(x, y)[1]]
-    return (lambda x:a * b**x), (a, b)
 
 def _isZero(x, fmt):
     "Check is text is formatted 0"
@@ -90,6 +40,47 @@ def _isZero(x, fmt):
     return x == fmt or x == "-" + fmt
 
 
+class Locus(_Locus):
+    "Class for drawing point sequences directly to the canvas"
+
+    def __init__(self, data, lrbt, param):
+        self.data = data
+        self.lrbt = lrbt
+        self.param = param
+        self.vars = {}
+
+    def _getCoordTr(self):
+        if self.lrbt is True:
+            cv = self.canvas
+            return lambda p: cv.px(*p)
+        else:
+            w, h = sz = self.canvas.size
+            return makeCS(self.lrbt, sz)[1]
+
+    def contains(self, pos): return False
+
+    def draw(self, srf, snapshot=False):
+        "Draw the locus to the sketch or canvas snapshot"
+        if snapshot: x0, y0 = 0, 0
+        else: x0, y0 = self.canvas.rect.topleft
+        tr = self._getCoordTr()
+        d = self.data
+        pts = d if type(d) in (list, tuple) else locus(d, self.param, **self.vars)
+        pts = [tr(p) for p in pts]
+        pts = [(x + x0, y + y0) for (x, y) in pts]
+        if len(pts) > 1:
+            wt = self.weight
+            return pygame.draw.lines(srf, self.stroke, False, pts, wt).inflate(wt, wt)
+        else: return pygame.Rect(0,0,0,0)
+
+    def pointGen(self):
+        "Generate a sequence of points using canvas pixel coordinates"
+        d = self.data
+        pts = d if type(d) in (list, tuple) else locus(d, self.param, **self.vars)
+        tr = self._getCoordTr()
+        for p in pts: yield tr(p)
+
+
 class Series:
     "Represents a single data series within the plot"
     _stroke = rgba((0, 0, 0))
@@ -208,11 +199,18 @@ class Plot(Renderable):
 #    contains = Image.contains
 
     def __init__(self, size, lrbt):
-        self._size = size
+        self._size = size.size if hasattr(size, "size") else size
         self.coords = _lrbt(lrbt, *size)
         self._keys = []
         self._series = {}
 
+    @property
+    def clockwise(self):
+        l, r, b, t = self._coords
+        x = 1 if r > l else -1
+        y = 1 if t > b else -1
+        return x * y < 0
+
     @property
     def size(self): return self._size
 
@@ -233,8 +231,17 @@ class Plot(Renderable):
         self._coords = _lrbt(lrbt, *self._size)
         w, h = self._size
         self.pixelCoords = coordTr(self._coords, [w-1, h-1])
+        self.plotCoords = coordTr(self._coords, [w-1, h-1], True)
         self.stale = True
 
+    @property
+    def units(self):
+        "Calculate plot scales"
+        px = self.pixelCoords
+        p0 = px((0, 0))
+        p1 = px((1, 1))
+        return p1[0] - p0[0], p1[1] - p0[1] 
+
     def __len__(self): return len(self._keys)
 
     def __setitem__(self, k, s):
@@ -337,49 +344,3 @@ class Plot(Renderable):
 
 
 class PlotSprite(Plot, BaseSprite): pass
-
-
-class Locus(Shape):
-    "Class for drawing point sequences directly to the canvas"
-    snapshot = None
-
-    def __init__(self, data, lrbt, param):
-        self.data = data
-        self.lrbt = lrbt
-        self.param = param
-        self.vars = {}
-
-    def _getCoordTr(self):
-        w, h = sz = self.canvas.size
-        return coordTr(_lrbt(self.lrbt, *sz), [w-1, h-1]) 
-
-    def contains(self, pos): return False
-
-    @property
-    def size(self):
-        return self.rect.size if hasattr(self, "rect") else (0,0)
-
-    def draw(self, srf, snapshot=False):
-        "Draw the locus to the sketch or canvas snapshot"
-        if snapshot: x0, y0 = 0, 0
-        else: x0, y0 = self.canvas.rect.topleft
-        tr = self._getCoordTr()
-        d = self.data
-        pts = d if type(d) in (list, tuple) else locus(d, self.param, **self.vars)
-        pts = [tr(p) for p in pts]
-        pts = [(x + x0, y + y0) for (x, y) in pts]
-        if len(pts) > 1:
-            wt = self.weight
-            return pygame.draw.lines(srf, self.stroke, False, pts, wt).inflate(wt, wt)
-        else: return pygame.Rect(0,0,0,0)
-
-    def pointGen(self):
-        "Generate a sequence of points using canvas pixel coordinates"
-        d = self.data
-        pts = d if type(d) in (list, tuple) else locus(d, self.param, **self.vars)
-        tr = self._getCoordTr()
-        for p in pts: yield tr(p)
-
-    @property
-    def pointList(self):
-        return list(self.pointGen())
