@@ -1,4 +1,4 @@
-# Copyright 2015-2019 D.G. MacCarthy <http://dmaccarthy.github.io>
+# Copyright 2015-2021 D.G. MacCarthy <http://dmaccarthy.github.io>
 #
 # This file is part of "sc8pr".
 #
@@ -18,13 +18,13 @@
 
 from math import hypot
 from pygame.sprite import collide_mask, collide_rect, collide_circle
-from sc8pr import BaseSprite, Image
+from sc8pr import BaseSprite, Image, Graphic
 
 
-class Sprite(BaseSprite):
-    "Sprite animation with one or more costumes"
+class CostumeImage(Graphic):
     costumeTime = 0
     _costumeNumber = 0
+    onreset = None
 
     def __init__(self, image, cols=1, rows=1, flip=0, padding=0):
         # Clone costumes images from existing list
@@ -75,12 +75,27 @@ class Sprite(BaseSprite):
         img.rect = self.rect
         return img.contains(pos)
 
-    def ondraw(self):
-        "Update sprite after drawing"
+    def updateCostume(self):
+        "Change sprite costume"
         n = self.costumeTime
+        if n < 0:
+            dn = -1
+            n = -n
+        else: dn = 1
         if n and self.sketch.frameCount % n == 0:
-            self.costumeNumber = self._costumeNumber + 1
-        super().ondraw()
+            self.costumeNumber = self._costumeNumber + dn
+            if self._costumeNumber == 0 and self.onreset: 
+                self.onreset()
+
+    ondraw = updateCostume
+
+
+class Sprite(CostumeImage, BaseSprite):
+    "Sprite animation with one or more costumes"
+
+    def ondraw(self):
+        BaseSprite.ondraw(self)
+        self.updateCostume()
 
 
 def collide_rect_mask(left, right):
@@ -104,12 +119,13 @@ class Collisions:
     def involving(self, sprite, group=None, asBool=False):
         "Detect whether a sprite is colliding with any of the specified sprites"
         coll = {}
-        for s in self._group(group):
-            if s is not sprite:
-                c = self.collide(sprite, s)
-                if c:
-                    if asBool: return True
-                    coll[s] = c
+        if hasattr(sprite, "rect"):
+            for s in self._group(group):
+                if s is not sprite:
+                    c = hasattr(s, "rect") and self.collide(sprite, s)
+                    if c:
+                        if asBool: return True
+                        coll[s] = c
         return False if asBool else coll
 
     def betweenMap(self, group1, group2=None):
