@@ -1,4 +1,4 @@
-# Copyright 2015-2019 D.G. MacCarthy <http://dmaccarthy.github.io>
+# Copyright 2015-2021 D.G. MacCarthy <http://dmaccarthy.github.io>
 #
 # This file is part of "sc8pr".
 #
@@ -84,6 +84,9 @@ class Font:
     def find(cls, *args):
         for f in args:
             f = cls._key(f)[0]
+            if f == "mono": f = Font.mono()
+            elif f == "sans": f = Font.sans()
+            elif f == "serif": f = Font.serif()
             if f in Font.installed(): return f
 
     @staticmethod
@@ -115,21 +118,25 @@ class Text(Renderable):
         return self
 
     def config(self, **kwargs):
-        keys = ("data", "color", "bg", "font", "fontSize", "fontStyle",
-            "height", "width", "align", "padding", "spacing", "weight",
-            "border", "promptColor")
+        keys = ("bg", "color", "border", "promptColor", "data",
+            "font", "fontSize", "fontStyle", "height", "width",
+            "align", "padding", "spacing", "weight")
         if hasAny(kwargs, keys): self.stale = True
         for a in kwargs:
             v = kwargs[a]
-            if v and a in ("bg", "color", "border", "promptColor"): v = rgba(v)
+            if v and a in keys[:4]: v = rgba(v)
             setattr(self, a, v)
         return self
 
+    @property
+    def renderer(self):
+        return Font.get(self.font, self.fontSize, self.fontStyle)
+
     def render(self):
         "Render the text as an Image"
         font = Font.get(self.font, self.fontSize, self.fontStyle)
         text = str(self.data).split("\n")
-        srfs = [font.render(t, True, self.color) for t in text]
+        srfs = [font.render(t if t else " ", True, self.color) for t in text]
         return self._joinLines(srfs)
 
     def _joinLines(self, srfs):
