@@ -1,4 +1,4 @@
-# Copyright 2015-2018 D.G. MacCarthy <http://dmaccarthy.github.io>
+# Copyright 2015-2022 D.G. MacCarthy <http://dmaccarthy.github.io>
 #
 # This file is part of "sc8pr".
 #
@@ -21,10 +21,19 @@ from random import randint
 from traceback import format_exc
 from zipfile import ZipFile
 from pathlib import Path
+from sc8pr.geom import sigma, delta
 
 scale = pygame.transform.smoothscale
 
 def nothing(*args): pass
+  
+def modKeys():
+    key = pygame.key.get_pressed()
+    m = 0
+    if key[pygame.K_LALT] or key[pygame.K_RALT]: m += 1
+    if key[pygame.K_LCTRL] or key[pygame.K_RCTRL]: m += 2
+    if key[pygame.K_LSHIFT] or key[pygame.K_RSHIFT]: m += 4
+    return m
 
 def resolvePath(rel, start=__file__, isDir=False):
     "Return an absolute path relative to a starting file or folder"
@@ -32,13 +41,20 @@ def resolvePath(rel, start=__file__, isDir=False):
     if not isDir: p = p.parent
     return str(p.joinpath(rel).resolve())
 
+def _pgc(c):
+    if not isinstance(c, pygame.Color):
+        try: c = pygame.Color(c)
+        except: c = pygame.Color(*c)
+    return c
+    
 def _rgba(*args):
     "Generate a sequence of pygame.Color instances"
     for c in args:
         t = type(c)
         if t is bool: # Random color
             c = [randint(0,255) for i in range(4 if c else 3)]
-        yield pygame.Color(c) if t is str else pygame.Color(*c)
+##        yield pygame.Color(c) if t is str else pygame.Color(*c)
+        yield _pgc(c)
 
 def rgba(*args):
     "Return a color or list of colors from str or tuple data"
@@ -99,23 +115,37 @@ def setAlpha(srf, a):
     srf.fill((255,255,255,a), special_flags=pygame.BLEND_RGBA_MIN)
     return srf
 
-def style(srf, bg=None, border=(0,0,0), weight=0, padding=0):
+def style(srf, bg=None, border=(0,0,0), weight=0, padding=0, borderradius=None):
     "Create a new surface with padding, background color, and/or border"
     w, h = srf.get_size()
 
     # Add padding
-    padding += weight
-    w += 2 * padding
-    h += 2 * padding
+    if type(padding) is int: px = py = padding + weight
+    else: px, py = [p + weight for p in padding]
+    w += 2 * px
+    h += 2 * py
     img = pygame.Surface((w, h), pygame.SRCALPHA)
 
     # Add background color and border
     if bg: img.fill(rgba(bg))
-    img.blit(srf, (padding, padding))
+    img.blit(srf, (px, py))
     if weight: drawBorder(img, border, weight)
+    if borderradius: img = borderRadius(img, borderradius)
 
     return img
 
+def borderRadius(srf, r=True):
+    w, h = srf.get_size()
+    if r is True: r = round(h / 6)
+    size = r, r
+    stamp = pygame.Surface(size, pygame.SRCALPHA)
+    pygame.draw.circle(stamp, (255,255,255), (0,0), r)
+    for p in [(w-r, h-r), (w-r, 0), (0, 0), (0, h-r)]:
+        srf.blit(stamp, p, special_flags=pygame.BLEND_RGBA_MIN)
+        stamp = pygame.transform.rotate(stamp, 90)
+    return srf
+# circle(surface, color, center, radius, width=0, draw_top_right=None, draw_top_left=None, draw_bottom_left=None, draw_bottom_right=None) -> Rect
+
 def drawBorder(srf, color=(0,0,0), weight=1, r=None):
     "Draw a border around the edges of the surface or specified rectangle"
     color = rgba(color)
@@ -147,9 +177,22 @@ def tile(srf, tile=0, cols=1, rows=1, padding=0):
 
 def ondrag(gr, ev):
     "Move a Graphic instance while dragging"
-    pos = gr.pos
-    dp = ev.rel
-    gr.pos = pos[0] + dp[0], pos[1] + dp[1]
+    gr.pos = sigma(gr.pos, ev.rel)
+
+def dragDrop(gr, ev):
+    "Drag a Graphic, possibly into a different canvas"
+    gr.hoverable = False
+    pos = sigma(gr.pos, ev.rel)
+    drop = False
+    try:
+        cv = [gr for gr in gr.sketch.objectAt(ev.pos).path if getattr(gr, "allowDrop", False)][0]
+        if cv is not gr.canvas:
+            pos = sigma(pos, delta(gr.canvas.rect.topleft, cv.rect.topleft))
+            drop = True
+            gr.setCanvas(cv)
+    except: pass
+    gr.config(pos=pos, hoverable=True)
+    if drop: gr.bubble("ondrop", ev)
 
 def fileExt(fn, ext):
     "Force file extension"
@@ -158,6 +201,17 @@ def fileExt(fn, ext):
         fn += "." + ext[0]
     return fn
 
+def mix(x, y):
+    "Generate a sequence of ordered pairs from two sequences"
+    try:
+        for i in x:
+            try:
+                for j in y: yield i, j
+            except:
+                for p in mix(i, [y]): yield p
+    except:
+        for p in mix([x], y): yield p
+
 
 class CachedSurface:
     "A class for caching scaled and rotated surfaces"
